@@ -3,17 +3,13 @@ import datetime
 import decimal
 import inspect
 import logging
+import netaddr
 import re
 import six
 import sys
 import uuid
 import weakref
 
-try:
-    import ipaddress
-except ImportError:
-    import ipaddr as ipaddress
-
 from wsme import exc
 
 log = logging.getLogger(__name__)
@@ -132,12 +128,19 @@ class BinaryType(UserType):
     def tobasetype(self, value):
         if value is None:
             return None
-        return base64.encodestring(value)
+        if six.PY3:
+            return base64.encodebytes(value)
+        else:
+            return base64.encodestring(value)
 
     def frombasetype(self, value):
         if value is None:
             return None
-        return base64.decodestring(value)
+        if six.PY3:
+            return base64.decodebytes(value)
+        else:
+            return base64.decodestring(value)
+
 
 #: The binary almost-native type
 binary = BinaryType()
@@ -218,7 +221,7 @@ class StringType(UserType):
             raise ValueError(error)
 
         if self.pattern is not None and not self.pattern.search(value):
-            error = 'Value should match the pattern %s' % self.pattern
+            error = 'Value should match the pattern %s' % self.pattern.pattern
             raise ValueError(error)
 
         return value
@@ -234,15 +237,19 @@ class IPv4AddressType(UserType):
     @staticmethod
     def validate(value):
         try:
-            ipaddress.IPv4Address(value)
-        except ipaddress.AddressValueError:
+            netaddr.IPAddress(value, version=4, flags=netaddr.INET_PTON)
+        except netaddr.AddrFormatError:
             error = 'Value should be IPv4 format'
             raise ValueError(error)
+        else:
+            return value
 
 
 class IPv6AddressType(UserType):
     """
     A simple IPv6 type.
+
+    This type represents IPv6 addresses in the short format.
     """
     basetype = six.string_types
     name = "ipv6address"
@@ -250,10 +257,12 @@ class IPv6AddressType(UserType):
     @staticmethod
     def validate(value):
         try:
-            ipaddress.IPv6Address(value)
-        except ipaddress.AddressValueError:
+            netaddr.IPAddress(value, version=6, flags=netaddr.INET_PTON)
+        except netaddr.AddrFormatError:
             error = 'Value should be IPv6 format'
             raise ValueError(error)
+        else:
+            return value
 
 
 class UuidType(UserType):
@@ -323,6 +332,7 @@ class UnsetType(object):
     def __repr__(self):
         return 'Unset'
 
+
 Unset = UnsetType()
 
 #: A special type that corresponds to the host framework request object.
@@ -485,7 +495,7 @@ class wsattr(object):
     def __set__(self, instance, value):
         try:
             value = validate_value(self.datatype, value)
-        except ValueError as e:
+        except (ValueError, TypeError) as e:
             raise exc.InvalidInput(self.name, value, six.text_type(e))
         dataholder = self._get_dataholder(instance)
         if value is Unset:
@@ -586,8 +596,8 @@ def inspect_class(class_):
             attrdef = attr
         else:
             if attr not in native_types and (
-                    inspect.isclass(attr)
-                    or isinstance(attr, (list, dict))):
+                    inspect.isclass(attr) or
+                    isinstance(attr, (list, dict))):
                 register_type(attr)
             attrdef = getattr(class_, '__wsattrclass__', wsattr)(attr)
 
@@ -739,6 +749,7 @@ class Registry(object):
             type_ = self.register(type_)
         return type_
 
+
 # Default type registry
 registry = Registry()
 
@@ -754,7 +765,7 @@ class BaseMeta(type):
         return type.__new__(cls, name, bases, dct)
 
     def __init__(cls, name, bases, dct):
-        if bases and bases[0] is not object:
+        if bases and bases[0] is not object and cls.__registry__:
             cls.__registry__.register(cls)
 
 
