@@ -2,15 +2,15 @@ from __future__ import absolute_import
 
 import functools
 import inspect
+import six
 import sys
 
+import pecan
+
 import wsme
 import wsme.rest.args
 import wsme.rest.json
 import wsme.rest.xml
-
-import pecan
-
 from wsme.utils import is_valid_code
 
 
@@ -43,6 +43,7 @@ class XMLRenderer(object):
             namespace['datatype']
         )
 
+
 pecan.templating._builtin_renderers['wsmejson'] = JSonRenderer
 pecan.templating._builtin_renderers['wsmexml'] = XMLRenderer
 
@@ -72,6 +73,8 @@ def wsexpose(*args, **kwargs):
 
         @functools.wraps(f)
         def callfunction(self, *args, **kwargs):
+            return_type = funcdef.return_type
+
             try:
                 args, kwargs = wsme.rest.args.get_args(
                     funcdef, args, kwargs, pecan.request.params, None,
@@ -85,9 +88,20 @@ def wsexpose(*args, **kwargs):
                 pecan.response.status = funcdef.status_code
                 if isinstance(result, wsme.api.Response):
                     pecan.response.status = result.status_code
+
+                    # NOTE(lucasagomes): If the return code is 204
+                    # (No Response) we have to make sure that we are not
+                    # returning anything in the body response and the
+                    # content-length is 0
+                    if result.status_code == 204:
+                        return_type = None
+                    elif not isinstance(result.return_type,
+                                        wsme.types.UnsetType):
+                        return_type = result.return_type
+
                     result = result.obj
 
-            except:
+            except Exception:
                 try:
                     exception_info = sys.exc_info()
                     orig_exception = exception_info[1]
@@ -106,20 +120,26 @@ def wsexpose(*args, **kwargs):
 
                 return data
 
-            if funcdef.return_type is None:
+            if return_type is None:
                 pecan.request.pecan['content_type'] = None
                 pecan.response.content_type = None
                 return ''
 
             return dict(
-                datatype=funcdef.return_type,
+                datatype=return_type,
                 result=result
             )
 
-        pecan_xml_decorate(callfunction)
-        pecan_text_xml_decorate(callfunction)
-        pecan_json_decorate(callfunction)
-        pecan.util._cfg(callfunction)['argspec'] = inspect.getargspec(f)
+        if 'xml' in funcdef.rest_content_types:
+            pecan_xml_decorate(callfunction)
+            pecan_text_xml_decorate(callfunction)
+        if 'json' in funcdef.rest_content_types:
+            pecan_json_decorate(callfunction)
+        if six.PY2:
+            argspec = inspect.getargspec(f)
+        else:
+            argspec = inspect.getfullargspec(f)
+        pecan.util._cfg(callfunction)['argspec'] = argspec
         callfunction._wsme_definition = funcdef
         return callfunction
 
